#!/usr/bin/python
# -*- coding: utf-8 -*-
# Python 2.7 Code

#   
#   _===_ _           _                  ______ _     _             _ _ _                 
#  |  ___| |         | |                 | ___ (_)   | |           (_) | |                
#  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __ 
#  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
#  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |   
#  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|   
#                                                                                         
# 
#  Fichier:   Interface_Charge_Fictive.py
#  Author: Électro-Bidouilleur   www.youtube.com
#  Mis a jour le 03.09.2015

from Tkinter import *      # Permet la création de fenêtres et de widgets
import tkMessageBox        # Permet l'utilisation d'une série de fenètres pré-définies (erreur, confirmation, sélection de fichier, etc.)
import time                # Permet l'accès à des fonctions de délai et d'heure
import thread              # Permet de rouler du code en multi-tâches
import serial              # Pyserial. Doit être installé au préalable. Voir http://pyserial.sourceforge.net/
from pychart import *

# VARIABLES GLOBALES
Vbatt = 0.00                           # La tension de la batterie
Ibatt = 0                              # Le courant de la batterie
Capacite_Batt = 0                      # La capacité calculée de la batterie en mAh
Ser_Char = ''                          # Caractère série reçu.
Empecher_Lecture_Periodique = False    # Sémaphore utilisé pour contrôler l'accès au port série
heure_depart = 0.0                     # L'heure de départ du cycle de décharge, exprimé en secondes
Cycle_Decharge_Actif = False           # Indicateur d'Activité du cycle de décharge
Alarme_Deja_Montree = False            # Indicateur pour vérifier si l'alarme a déjà été montrée
data = []
index_data = 0
Liste_Etat = []                        # La réponse de la commande d'état, brisée en liste

# La liste des valeurs de courant de décharge qui sera contenue dans le menu déroulant
Liste_Courant_Dech = [
   "100","200","300","400","500","600","700","800","900", "1000",
   "1100","1200","1300","1400","1500","1600","1700","1800","1900", "2000",
   "2100","2200","2300","2400","2500","2600","2700","2800","2900", "3000"
]

# La liste des valeurs de tension d'arrêt qui sera contenue dans le menu déroulant
Liste_Tension_Arret = [
   "1.0", "1.5","2.0", "2.5", "3.0", "3.5","4.0", "4.5", "5.0", "5.5","6.0", "6.5",
   "7.0", "7.5","8.0", "8.5", "9.0", "9.5","10.0", "10.5", "11.0", "11.5","12.0", "12.5",
   "13.0", "13.5","14.0", "14.5", "15.0", "15.5","16.0", "16.5", "17.0", "17.5","18.0", "18.5"
]

def fonction_lecture_etat():
   global Liste_Etat
   global ser
   ser.flushInput() # Vider la mémoire tampon de réception du port série
   ser.write("E0000\n") # Oui, Envoyer la commande de demande d'état au PIC     
   time.sleep(0.5)
   Ser_Char = ''             # Initialiser le caractère Rx série avec un espace
   result = ''                # Initialiser la chaîne de caractères Rx série à vide
   while (Ser_Char <> "\0"):  # Lire des caractères du port série jusqu'à la réception du caractère NULL (ASCII 0)
      if (ser.inWaiting() > 0):
         Ser_Char = ser.read(1)        # Attendre un caractère et le lire
         result = result + Ser_Char    # Concaténation à la chaîne de caractères
      else:          # Gestion d'erreur
         ser.close()       # Fermer le port série
         tkMessageBox.showerror("Charge Ficttive C.C.","ERREUR !!!  Perdu la communication avec le système.")
         Fenetre_Princ.destroy()    # Quitter le programme
   Liste_Etat = result.split(',')      # séparation des différents champs de la commande d'état


def courbe_decharge(): 
   global data
   global Capacite_Batt
   
   can = canvas.init(fname="./Courbe_Décharge.pdf", format = "PDF")   
   theme.get_options()
   theme.scale_factor = 1
   theme.reinitialize() # Must be called so that the above variables get updated
   xaxis = axis.X(format="/a-90/hL%d", tic_interval = 300, tic_len = 10, minor_tic_interval = 60, minor_tic_len = 5, label=u"/14Temps (Secondes)")
   yaxis = axis.Y(tic_interval = 5, label=u"/14Tension (Volts)")
   ar = area.T(x_axis=xaxis, y_axis=yaxis, y_range=(0,20), size=((1000,600)))
   plot = line_plot.T(data=data, ycol=1, tick_mark=tick_mark.star3)
   ar.add_plot(plot)
   ar.draw()
   label_temp_courant = u"/20Courant de Décharge: " + Courant_Dech_Sel.get() + " mA"
   label_temp_capacite = u"/20Capacité Calculée: " + str(round(Capacite_Batt,0))[:-2] + " mAh"
   tbox1 = text_box.T(loc=(700,570), text=label_temp_courant, line_style=None)
   tbox1.draw()
   tbox2 = text_box.T(loc=(700,540), text=label_temp_capacite, line_style=None)
   tbox2.draw()

   can.close()

# Cette routine est appelée lorsque le bouton de marche/arrêt est actionné
def Bascule_Bouton_Demarrage():
   global Vbatt
   global Ibatt
   global Capacite_Batt
   global Ser_Char
   global Empecher_Lecture_Periodique
   global heure_depart
   global Cycle_Decharge_Actif
   global data
   global ser
   global Liste_Etat 

   Empecher_Lecture_Periodique = True           # Monter le sémaphore d'utilisation du port série
   # Traiter l'arrêt du cycle de décharge
   if Bouton_Demarrage.config('relief')[-1] == 'sunken':       # Bouton enfoncé?
      # Mettre à jour l'état des boutons et le texte           # Oui
      Bouton_Demarrage.config(relief="raised")                 # Soulever le bouton
      Bouton_Demarrage.config(text="Démarrage")                # Changer le texte
      Courant_Dech_Menu.config(state='normal')                 # remettre actif les deux menus déroulants
      Tension_Arret_Menu.config(state='normal')                #                 "
      Cycle_Decharge_Actif = False                             # Changer le statut du cycle de décharge
      data = []                     # Effacer la liste de points
      # Stopper le cycle de décharge 
      ser.write("S0000\n")             # Commande pour interrompre le cycle de décharge
      
   # Traiter le démarrage du cycle de décharge
   else:    # Bouton soulevé
      # Mettre à jour l'état des boutons et le texte
      Bouton_Demarrage.config(relief="sunken")                 # Enfoncer bouton
      Bouton_Demarrage.config(text="Arrêt")                    # Changer le texte
      Courant_Dech_Menu.config(state='disabled')               # Désactiver les 2 menus déroulants
      Tension_Arret_Menu.config(state='disabled')              #               "
      Cycle_Decharge_Actif = True                              # Activer le cycle de décharge
       
      # Configurer la charge fictive
      Ibatt_Cible = round((float(Courant_Dech_Sel.get()) / 1000 * 1023 * 2 / 5 / 4), 0)       # Calculer le paramètre de courant cible à envoyer au PIC
      time.sleep(1.5)     # Pause de 0.6 seconde. Garantit que la lecture périodique est complétée une fois l'indicateur Empecher_Lecture_Periodique = True
      ser.write("C" + str(Ibatt_Cible)[:-2].zfill(4) + "\n")   # Envoyer la commande de courant cible formattée correctement
      time.sleep(0.1)      # Petite pause
      Tension_Arret = round((float(Tension_Arret_Sel.get()) / 4 * 1023 / 5), 0)       # Calculer le paramètre de tension d'arrêt à envoyer au PIC
      ser.write("A" + str(Tension_Arret)[:-2].zfill(4) + "\n")    # Envoyer la commande de tension d'arrêt formattée correctement
      time.sleep(0.1)      # Petite pause
      heure_depart = time.time()    # Capture de l'heure courant comme heure de départ, exprimée en secondes
      Capacite_Batt = 0       # Initialisation de la capacité de la batterie
      # Démarrer le cycle de décharge 
      ser.write("G0000\n") # Commande pour démarrer le cycle de décharge
      while (True):     # Boucle perpétuelle interrompue par le "break"
         fonction_lecture_etat()          # Lire l'état du système...
         if (Liste_Etat[1] == '1'): break    # ... jusqu'à ce que le PIC montre le cycle de décharge comme actif.
   Empecher_Lecture_Periodique = False           # Descendre le sémaphore d'utilisation du port série         
         
# Tâche (thread) exécutée en parallàle à la boucle de la fenêtre principale.
# Gère la lecture de l'état de la charge fictive et dérive les tensions et courants du moment.           
def Lecture_Etat_Periodique():
   global Vbatt
   global Ibatt
   global Capacite_Batt
   global Ser_Char
   global Empecher_Lecture_Periodique
   global Label_Tension_Batterie
   global Label_Courant_Decharge
   global heure_depart
   global Cycle_Decharge_Actif
   global data
   global Liste_Etat 
   
   while(1):   # Boucle perpétuelle
      if (Empecher_Lecture_Periodique == False):    #Port ouvert et sémaphore de port série OK?
         fonction_lecture_etat()
         print Liste_Etat
         # Calcul de la tension de batterie
         Vbatt = float(Liste_Etat[4]) / 1023 * 5 * 4     # Calcul de la tension de battereie en Volts
         Chaine_tension_batt.set(str(round(Vbatt,2)))    # Afficher la tension de batterie
            
         # Gestion des alarmes.
         if (Liste_Etat[2] == '1'):       # Alarme active?
            tkMessageBox.showerror("Problème Détecté", "La Charge Fictive ne peut générer le courant demandé...")    # Non, afficher fenêtre d'alarme
            Couleur_Alarme.set('red')     # Oui, Mettre l'indicateur d'alarme à rouge   
            Indic_Alarme.config(bg=Couleur_Alarme.get())    #   "
            # Mettre à jour l'état des boutons et le texte    
            Bouton_Demarrage.config(relief="raised")                 # Soulever le bouton
            Bouton_Demarrage.config(text="Démarrage")                # Changer le texte
            Courant_Dech_Menu.config(state='normal')                 # remettre actif les deux menus déroulants
            Tension_Arret_Menu.config(state='normal')                #                 "
            ser.write("D0000\n")       # Envoyer la commande de d'effaçage d'alarme au PIC     
         elif (Liste_Etat[2] == '0'):     # Alarme inactive?
            Couleur_Alarme.set('green')     # Oui, Mettre l'indicateur d'alarme à rouge   
            Indic_Alarme.config(bg=Couleur_Alarme.get())    #   "

         # Gestion des valeurs calculées
         if (Cycle_Decharge_Actif):    # Cycle de décharge actif?
            #  Oui, Tester l'arrêt automatique...
            if (Liste_Etat[1] == '0'):       #Indicateur de cycle de décharge désactivé?
               Cycle_Decharge_Actif = False     # Oui, désactiver le cycle de décharge              
               # Mettre à jour l'état des boutons et le texte    
               Bouton_Demarrage.config(relief="raised")                 # Soulever le bouton
               Bouton_Demarrage.config(text="Démarrage")                # Changer le texte
               Courant_Dech_Menu.config(state='normal')                 # remettre actif les deux menus déroulants
               Tension_Arret_Menu.config(state='normal')                #                 "
               tkMessageBox.showinfo("Charge Ficttive C.C.","Cycle de décharge terminé!")
            else:       # Cycle de décharge actif, mêttre à jour le courant et la capacité.
               Ibatt = float(Liste_Etat[3]) / 1023 * 5 * 4 / 2 *1000    # Calculer le courant de décharge du moment
               Chaine_courant_decharge.set(str(round(Ibatt,0))[:-2])    # Afficher le courant de décharge
               heure_courante = time.time() - heure_depart
               Capacite_Batt = float(Courant_Dech_Sel.get()) * heure_courante / 3600    # Calculer la capacité de la batterie
               Chaine_capacite_batt.set(str(round(Capacite_Batt,0))[:-2])     # Afficher la capacité de la batterie
               # Sauvegarde de la donnée dans le fichier de résultats
               fichier_sauv_donnees = open('./donnees_sauvegardees.txt','a')                  # Ouvrir le fichier de sauvegarde des données
               fichier_sauv_donnees.write(str(round(heure_courante,0))[:-2] + "," + str(round(Vbatt,2)) + "," + str(round(Capacite_Batt,0))[:-2] + "\n")    # Sauvegarder la ligne de données
               fichier_sauv_donnees.close()                                    # Fermer le fichier de sauvegarde
               # Production du graphique
               data.extend([(round(heure_courante),Vbatt)])       # ajouter la lecture de tension à la liste de points             
               courbe_decharge()       # Invoquer la fonction de producton de courbe
         else: Chaine_courant_decharge.set('-')      # Décharge inactive, afficher - au lieu du courant de décharge
      time.sleep(1.5)     # Pause de 2 secondes avant de boucler
   
   
# CODE PRINCIPAL   
#Création de la fenêtre principale
Fenetre_Princ = Tk()
Fenetre_Princ.title("ÉLectro-Bidouilleur")     # Titre donné à la fenêtre
#Fenetre_Princ.iconbitmap("@"+"Bid_Logo_No_Background_64x64.xbm")
Fenetre_Princ.geometry('{}x{}'.format(420, 280))

Label(Fenetre_Princ, text="Contrôle de Charge Fictive C.C.", font=("Helvetica", 16),fg="blue").place(x=50, y=7)  # Etiquette du Titre

Label(Fenetre_Princ, text="Courant de Décharge (mA): ").place(x=50, y=50)    

# Création du menu déroulant de choix de courant de décharge
Courant_Dech_Sel = StringVar(Fenetre_Princ)     # Créer la variable texte
Courant_Dech_Sel.set("100")      # Lui donner la valeur initiale                     
Courant_Dech_Menu = OptionMenu(Fenetre_Princ, Courant_Dech_Sel, *Liste_Courant_Dech)      # Créer le menu déroulant
Courant_Dech_Menu.config(width=5)
Courant_Dech_Menu.place(x=300, y=45)     # Positionner le menu déroulant

Label(Fenetre_Princ, text="Tension d'Arrêt (V):      ").place(x=50, y=80)  # Etiquette de la tension d'arrêt

# Création du menu déroulant de choix de tension d'arrêt
Tension_Arret_Sel = StringVar(Fenetre_Princ)      # Créer la variable texte
Tension_Arret_Sel.set("9.0")      # Lui donner la valeur initiale    
Tension_Arret_Menu = OptionMenu(Fenetre_Princ, Tension_Arret_Sel, *Liste_Tension_Arret)      # Créer le menu déroulant
Tension_Arret_Menu.config(width=5)
Tension_Arret_Menu.place(x=300, y=75)      # Positionner le menu déroulant

# Création du bouton de démarrage/arrêt du cycle de décharge
Bouton_Demarrage = Button(Fenetre_Princ, text="Démarrage", width=20, relief="raised", command=Bascule_Bouton_Demarrage)     #Créer le bouton à bascule
Bouton_Demarrage.place(x=130, y=210)    # Positionner le bouton

# Création de l'indicateur d'alarme
Couleur_Alarme = StringVar(Fenetre_Princ)     # Créer la variable texte     
Couleur_Alarme.set('grey')       # Assigner la couleur initiale
Indic_Alarme = Label(Fenetre_Princ, width=2, bg=Couleur_Alarme.get())      # Créer l'indicateur d'alarme
Indic_Alarme.place(x=80, y=215)     # Positionner l'indicateur.

Label(Fenetre_Princ, text="Tension Batterie (V):      ").place(x=50, y=120)  # Etiquette de la tension de batterie

Chaine_tension_batt = StringVar()     # Créer la variable texte   
Chaine_tension_batt.set("-")      # Lui donner la valeur initiale        
Label_Tension_Batterie = Label(Fenetre_Princ, textvariable=Chaine_tension_batt).place(x=320, y=120)  # Etiquette de la valeur de la tension de la batrerie

Label(Fenetre_Princ, text="Courant Décharge (mA):     ").place(x=50, y=150)      # Etiquette du courant de décharge

Chaine_courant_decharge = StringVar()     # Créer la variable texte     
Chaine_courant_decharge.set("-")     # Assigner la valeur initiale
Label_Courant_Decharge = Label(Fenetre_Princ, textvariable=Chaine_courant_decharge).place(x=320, y=150)  # Etiquette de la valeur de du courant de décharge

Label_Menu_Capacite_texte = StringVar()
Label_Menu_Capacite_texte.set("Capacité Batterie (mAh):     ")
Label_Menu_Capacite = Label(Fenetre_Princ, textvariable=Label_Menu_Capacite_texte).place(x=50, y=180)      # Etiquette de la capacité de la batterie

Chaine_capacite_batt = StringVar()     # Créer la variable texte     
Chaine_capacite_batt.set("-")     # Assigner la valeur initiale
Capacite_Batterie = Label(Fenetre_Princ, textvariable=Chaine_capacite_batt).place(x=320, y=180)  # Etiquette de la valeur de de la capacité de la batterie


# Ouverture du port série
try:  # Tentative
   print "En attente de l'ouverture du port série..."   # Message affiché
   ser = serial.Serial("/dev/ttyUSB0", 2400, timeout=1, xonxoff=False)     # Configurer le port série
   ser.flushInput() # Vider la mémoire tampon de réception du port série
except:  # Problème d'ouverture du port série
   print("ERREUR !!!  Impossible d'ouvrir le port série. Au revoir.")
   time.sleep(2)  # Pause 
   sys.exit()     # Quitter le programme
print "Port série ouvert."   # Message affiché

#Essai de lecture du port série pour vérifier la présence du PIC
result =""                       # Initialiser les tampons de lecture
Ser_Char = ""                    #        "
ser.write("E0000\n")          # Envoyer la commande de demande d'état au PIC
time.sleep(0.5)
while (Ser_Char <> "\0"):     # Lire des caractères du port série jusqu'à la réception du caractère NULL (ASCII 0)
   if (ser.inWaiting() > 0):
      Ser_Char = ser.read(1)        # Attendre un caractère et le lire
      result = result + Ser_Char    # Concaténation à la chaîne de caractères
   else:          # Gestion d'erreur
      ser.close()       # Fermer le port série
      print("ERREUR !!!  La Charge Fixctive ne répond pas. Au revoir.")    # Imprimer un messaga d'erreur
      sys.exit()     # Quitter le programme

# Port série actif 
Couleur_Alarme.set('green')                        # Mettre l'indicateur d'alarme en rouge
Indic_Alarme.config(bg=Couleur_Alarme.get())       #        "
thread.start_new_thread(Lecture_Etat_Periodique, ())  # Démarrage de la tâche de lecture de l'état du PIC
Fenetre_Princ.mainloop()      
